program objectiveValuation;
uses kantor,csvfilereader;
(*!
\section{Purpose}
The programme applies the Objective Valuation technique of the Soviet Nobelist Kantorovich to data
from input output tables. Kantorovich derived a set of numbers, which he termed Objectively
Determined Valuations, from a combination of
\begin{enumerate}
\item the technology available
\item the objectives of the socialist plan
\item the available stock of material resources
\end{enumerate}

This programme can potentially be used in studies of the comparative efficiency of labour values,
prices of production or Kantorovich valuations in economic planning.
\section{Comand line interface}
{\bf Usage :} {\tt objectiveValuation \em iotable.csv planray.csv resourcevec.csv $>$ result.csv}


The files should be laid out with a first line and first column made up of text fields labeling
the rows and columns. All other cells should be numeric.
Discussion of matrix sizes in what follows refers exclusively to the rectangular subarray of
numeric values.
\subsection{iotable}
The first file, the iotable one, should contain an N by M io table in standard column form,
with a column corresponding to an industry so that cell at row i col j contains the amount
of output from the ith industry used by the jth industry.

The last line must contain the outputs of each industry.
\subsection{planray}
This contains the target level of gross outputs in the next period to be obtained using the
technology of the iotable. In line with Kantorovich practice this corresponds to a ray
in N-1 dimensional space along which the outputs are supplied in fixed proportions. This is also
sometimes called a Leontief demand function.
The format, excluding headers, is a 1 by N-1 matrix, or N-1 element row vector.
For a typical io table this implies that even though labour is not a produced input
it should appear as a zero output target in the plan.

Headers should correspond to the column headers of the IO table.

\subsection{reseourcevec}
This is a N-1 row vector of available resources, the headers should correspond to the row titles of the IO table.

\subsection{results}
The output is a 2 by M matrix in csv format. The two rows encode the intensity with which each
column technology in the IO table must be operated and the objective valuations associated with
each output.


 *)
 type pmat=^matrix;
      channel= record
		p:pcsv;
		r,c:pheadervec;
		m:^matrix;
	  end;
procedure rf(var ch:channel;param:integer );
(*! Read in one of the file parameters and extract the data from it *)
begin
    with ch do 
    begin
		p:=parsecsvfile(paramstr(param));
		if p=nil then 
		begin
		   writeln('error opening or parsing file ',paramstr(param));
		   halt(2);
		end
		else ;
		r:=getrowheaders(p);
		c:=getcolheaders(p);
		m:=getdatamatrix(p);
	end;
end;
var iot,rv,pr,results:channel; 
    tech:^technologytable;  odvs:^ODVvec;   X:^intensityvec;
    target,startingresource:^resourcevec;
    outputs:^resourcevec;
    i,j:integer;
begin
	rf(iot,1);
	rf(pr,2);
	rf(rv,3);
	
	new(tech,iot.m^.cols,iot.m^.rows-2);{omit last row and allow for the base of 2nd dim of techniques to be 0 }
	tech^:=0;
	
	for i := 1 to tech ^.omega do
	begin
 		for j := agriculture to tech ^.amusements do
		begin
			tech^[i,src,j]:= -iot.m^[j+1,i];{ inputs go in transposed with minus sign }					
		end;
		tech^[i,dest,i-1]:= iot.m^[iot.m^.rows,i];{ outputs from last row of io matrix}
	end;
		
	new(target,pr.m^.cols-1);
	target^:=pr.m^[1,1+iota 0];
	new(startingresource,rv.m^.cols-1);
	new(odvs,rv.m^.cols-1); { there is to be an odv for each resource }
	new(X,iot.m^.cols);{ there is an intensity for each production technique, thus 1 for each column in the io table}
	startingresource^:=rv.m^[1,1+iota  0];
	optimiser:=pauls;
	kantorovichMethod(tech^,target^ ,  
	startingresource^,
	 odvs^, X ^ );	

	  for i:=1 to iot.m^.cols do
	 write( ',',iot.c^[i]^.textual^);
	writeln;
	write('output');
	 for i:=1 to iot.m^.cols do
	 write(',',x^[i]* iot.m^[iot.m^.rows,i]:1:4);writeln;
	 write('odv');
	 for i:=1 to iot.m^.cols do
	 write(',',odvs^[i-1]:1:4);
	 writeln;
	
	 
	
	 
	 
end.

