program labourValuation;
uses csvfilereader;
(*!
\section{Purpose}
The programme applies the labour value theory to data
from input output tables. 

This programme can potentially be used in studies of the comparative efficiency of labour values,
prices of production or Kantorovich valuations in economic planning.
\section{Comand line interface}
{\bf Usage :} {\tt labourValuation \em iotable.csv }


The files should be laid out with a first line and first column made up of text fields labeling
the rows and columns. All other cells should be numeric.
Discussion of matrix sizes in what follows refers exclusively to the rectangular subarray of
numeric values.
\subsection{iotable}
The first file, the iotable one, should contain an N by M io table in standard column form,
with a column corresponding to an industry so that cell at row i col j contains the amount
of output from the ith industry used by the jth industry.

The last line must contain the outputs of each industry.



 *)
 type pmat=^matrix;
  channel= record
    p:pcsv;
    r,c:pheadervec;
    m:^matrix;
   end;
procedure rf(var ch:channel;param:integer );
(*! Read in one of the file parameters and extract the data from it *)
begin
 with ch do 
 begin
    p:=parsecsvfile(paramstr(param));
    if p=nil then 
    begin
     writeln('error opening or parsing file ',paramstr(param));
     halt(2);
    end
    else ;
    r:=getrowheaders(p);
    c:=getcolheaders(p);
    m:=getdatamatrix(p);
  end;
end;
var iot:channel; 
  L,V,O,O1:^vector;
  square:^matrix; 
 i,j:integer;
begin
  rf(iot,1);
  new(L,iot.m^.cols);new(V,iot.m^.cols);new(O,iot.m^.cols);new(O1,iot.m^.cols);
  {initialise all labour values to 1}
  L^:=1;  
  { extract the variable capital vector from the IO table}    
  V^:=iot.m^ [iot.m^.rows-1]; 
  {Extract the final output price vector from the IO table}
  O^:=iot.m^ [iot.m^.rows]; 
  {Create a transposed version of the IO table central matrix} 
  new(square,iot.m^.cols,iot.m^.cols);square^ := trans (iot.m^); 
  {Jacobi solution of labour values, 20 iterations is enough to give a very accurate estimate }
  for i:= 1 to 20 do
  begin
    o1^:= V^+ \+ (L^ * square ^);
    L^:= O1^/O^;
  end;
  writeln(O1^:12:1,O^:12:1);
end.

