program multitechnique;
uses kantor,csvfilereader;
(*!
\section{Purpose}
The programme applies the Objective Valuation technique of the Soviet Nobelist Kantorovich to data
from input output tables. Kantorovich derived a set of numbers, which he termed Objectively
Determined Valuations, from a combination of
\begin{enumerate}
\item the technology available
\item the objectives of the socialist plan
\item the available stock of material resources
\end{enumerate}

This programme can potentially be used in studies of the comparative efficiency of labour values,
prices of production or Kantorovich valuations in economic planning.
\section{Comand line interface}
{\bf Usage :} {\tt multitechnique \em iotable.csv planray.csv resourcevec.csv $>$ result.csv}


The files should be laid out with a first line and first column made up of text fields labeling
the rows and columns. All other cells should be numeric.
Discussion of matrix sizes in what follows refers exclusively to the rectangular subarray of
numeric values.
\subsection{iotable}

The first file, the iotable one, should contain an 2 M row by N  column io table in standard column form,
with a column corresponding to an technique so that cell at row i col j contains the amount
of  the ith product used by the jth technique, and the cell at position [i+M,j] specifies the amount
 of the ith product produced by the jth technique.

\subsection{planray}
This contains the target level of gross outputs in the next period to be obtained using the
technology of the iotable. In line with Kantorovich practice this corresponds to a ray
in N-1 dimensional space along which the outputs are supplied in fixed proportions. This is also
sometimes called a Leontief demand function.
The format, excluding headers, is a 1 by N-1 matrix, or N-1 element row vector.
For a typical io table this implies that even though labour is not a produced input
it should appear as a zero output target in the plan.

Headers should correspond to the column headers of the IO table.

\subsection{reseourcevec}
This is a N-1 row vector of available resources, the headers should correspond to the row titles of the IO table.

\subsection{results}
The output is a 2 by M matrix in csv format. The two rows encode the intensity with which each
column technology in the IO table must be operated and the objective valuations associated with
each output.


 *)
 type pmat=^matrix;
      channel= record
		p:pcsv;
		r,c:pheadervec;
		m:^matrix;
	  end;
procedure rf(var ch:channel;param:integer );
(*! Read in one of the file parameters and extract the data from it *)
begin
    with ch do 
    begin
		p:=parsecsvfile(paramstr(param));
		if p=nil then 
		begin
		   writeln('error opening or parsing file ',paramstr(param));
		   halt(2);
		end
		else ;
		r:=getrowheaders(p);
		c:=getcolheaders(p);
		m:=getdatamatrix(p);
	end;
end;
var iot,rv,pr,results:channel; 
    tech:^technologytable;  odvs:^ODVvec;   X:^intensityvec;
    target,startingresource:^resourcevec;
    outputs:^resourcevec;
    i,j:integer;
begin
	rf(iot,1);
	rf(pr,2);
	rf(rv,3);
{	writeln ('iotable size',iot.m^.cols,iot.m^.rows  );{}
	new(tech,iot.m^.cols,(iot.m^.rows div 2 )-1); 
	tech^:=0;
{	writeln('techniqueN', tech^.techniqueN, ' resourceM ', tech^.resourcem);{}
	for i := technique1 to tech ^.techniqueN do
	begin
	{    writeln('technique ',i);}
 		for j := resource0 to tech ^.resourceM do
		begin
		{    writeln('resource ',j);
		    	writeln('write to pos ',i,src,j, ' read from ',j+1 ,i);}
			tech^[i,src,j]:= -iot.m^[j+1,i];{ inputs go in transposed with minus sign }	
		{ 	writeln('write',iot.m^[j+2+tech^.resourceM,i],' to pos ',i:4,dest:2,j:4, ' read from ',j+1+tech^.resourceM,i);{}
			tech^[i,dest,j]:= iot.m^[j+2+tech^.resourceM,i]; 				
		end;
		
	end;
		
	new(target,pr.m^.cols-1);
	target^:=pr.m^[1,1+iota 0];
	new(startingresource,rv.m^.cols-1);
	new(odvs,rv.m^.cols-1); { there is to be an odv for each resource }
	new(X,iot.m^.cols);{ there is an intensity for each production technique, thus 1 for each column in the io table}
	startingresource^:=rv.m^[1,1+iota  0];
	optimiser:=pauls;
	kantorovichMethod(tech^,target^ ,  	startingresource^,	 odvs^, X ^ );	
	 
	 
	 writeln('intensities');
	 for i:=1 to iot.m^.cols do write( ',',iot.c^[i]^.textual^);
	 writeln;
	 for i:=1 to x^.cols do write( ',',x^[i]);
	 writeln;
	 writeln('odvs');
	 for i:=0 to odvs^.resourceM do write( ',',iot.r^[i+1]^.textual^);
	 writeln;
	 
	 for i:=0 to odvs^.resourceM do write(',',odvs^[i]);
	
	 writeln;
	
	 
	 
end.

